@@ -1,11 +1,13 @@
 import os
 import sys
+import glob
+import shutil
 from pythonbrew.basecommand import Command
 from pythonbrew.define import PATH_PYTHONS, PATH_VENVS, PATH_HOME_ETC_VENV,\
     PATH_ETC, VIRTUALENV_DLSITE, PATH_DISTS
 from pythonbrew.util import Package, \
     is_installed, get_installed_pythons_pkgname, get_using_python_pkgname,\
-    untar_file, Subprocess, rm_r, copy_libs
+    untar_file, Subprocess, rm_r
 from pythonbrew.log import logger
 from pythonbrew.downloader import Downloader
 
@@ -23,13 +25,6 @@ class VenvCommand(Command):
             help="Use the specified version of python.",
             metavar='VERSION'
         )
-        self.parser.add_option(
-            "-a", "--all",
-            dest="all",
-            action='store_true',
-            default=False,
-            help="Show the all python environments.",
-        )
         self.parser.add_option(
             "-n", "--no-site-packages",
             dest="no_site_packages",
@@ -39,6 +34,7 @@ class VenvCommand(Command):
         )
         self._venv_dir = os.path.join(PATH_ETC, 'virtualenv')
         self._venv = os.path.join(self._venv_dir, 'virtualenv.py')
+        self._clear()
         
     def run_command(self, options, args):
         if not args:
@@ -61,15 +57,13 @@ class VenvCommand(Command):
                 logger.error('%s is not installed.' % pkgname)
                 sys.exit(1)
         else:
-            # check using python under pythonbrew
             pkgname = get_using_python_pkgname()
-            if not pkgname:
-                logger.error('Can not use venv command before switching a python.  Try \'pythonbrew switch <version of python>\'.')
-                sys.exit(1)
+        
         self._pkgname = pkgname
-        self._target_py = os.path.join(PATH_PYTHONS, pkgname, 'bin', 'python')
-        self._workon_home = os.path.join(PATH_VENVS, pkgname)
-        self._py = os.path.join(PATH_PYTHONS, pkgname, 'bin', 'python')
+        if self._pkgname:
+            self._target_py = os.path.join(PATH_PYTHONS, pkgname, 'bin', 'python')
+            self._workon_home = os.path.join(PATH_VENVS, pkgname)
+            self._py = os.path.join(PATH_PYTHONS, pkgname, 'bin', 'python')
         
         # is already installed virtualenv?
         if not os.path.exists(self._venv):
@@ -80,8 +74,8 @@ class VenvCommand(Command):
     
     def run_command_init(self):
         if os.path.exists(self._venv):
-            logger.info('venv command is already initialized.')
-            return
+            logger.info('Remove virtualenv. (%s)' % self._venv_dir)
+            rm_r(self._venv_dir)
         if not os.access(PATH_DISTS, os.W_OK):
             logger.error("Can not initialize venv command: Permission denied.")
             sys.exit(1)
@@ -90,34 +84,15 @@ class VenvCommand(Command):
         d.download('virtualenv.tar.gz', VIRTUALENV_DLSITE, download_file)
         logger.info('Extracting virtualenv into %s' % self._venv_dir)
         untar_file(download_file, self._venv_dir)
-    
-    def run_command_rename(self, options, args):
-        if len(args) < 3:
-            logger.error("Unrecognized command line argument: ( 'pythonbrew venv rename <source> <target>' )")
-            sys.exit(1)
-            
-        if not os.access(PATH_VENVS, os.W_OK):
-            logger.error("Can not rename a virtual environment in %s.\nPermission denied." % PATH_VENVS)
-            sys.exit(1)
         
-        source_dir = os.path.join(self._workon_home, args[1])
-        target_dir = os.path.join(self._workon_home, args[2])
-        
-        if not os.path.isdir(source_dir):
-            logger.error('%s does not exist.' % source_dir)
-            sys.exit(1)
-            
-        if os.path.isdir(target_dir):
-            logger.error('Can not overwrite %s.' % target_dir)
-            sys.exit(1)
-            
-        os.rename(source_dir, target_dir)
-    
     def run_command_create(self, options, args):
         if not os.access(PATH_VENVS, os.W_OK):
             logger.error("Can not create a virtual environment in %s.\nPermission denied." % PATH_VENVS)
             sys.exit(1)
-
+        if not self._pkgname:
+            logger.error("Unknown python version: ( 'pythonbrew venv create <project> -p VERSION' )")
+            sys.exit(1)
+        
         virtualenv_options = []
         if options.no_site_packages:
             virtualenv_options.append('--no-site-packages')
@@ -132,15 +107,87 @@ class VenvCommand(Command):
             # create environment
             s = Subprocess(verbose=True)
             s.call(cmd)
+    
+    def run_command_delete(self, options, args):
+        if not self._pkgname:
+            logger.error("Unknown python version: ( 'pythonbrew venv delete <project> -p VERSION' )")
+            sys.exit(1)
         
+        for arg in args[1:]:
+            target_dir = os.path.join(self._workon_home, arg)
+            if not os.path.isdir(target_dir):
+                logger.error('%s does not exist.' % target_dir)
+            else:
+                if not os.access(target_dir, os.W_OK):
+                    logger.error("Can not delete %s.\nPermission denied." % target_dir)
+                    continue
+                logger.info('Deleting `%s` environment in %s' % (arg, self._workon_home))
+                # make command
+                rm_r(target_dir)
+    
+    def run_command_use(self, options, args):
+        if len(args) < 2:
+            logger.error("Unrecognized command line argument: ( 'pythonbrew venv use <project>' )")
+            sys.exit(1)
+        
+        workon_home = None
+        activate = None
+        if self._pkgname:
+            workon_home = self._workon_home
+            activate = os.path.join(workon_home, args[1], 'bin', 'activate')
+        else:
+            for pkgname in get_installed_pythons_pkgname():
+                workon_home = os.path.join(PATH_VENVS, pkgname)
+                if os.path.isdir(workon_home):
+                    if len([d for d in os.listdir(workon_home) if d == args[1]]) > 0:
+                        activate = os.path.join(workon_home, args[1], 'bin', 'activate')
+                        break
+        if not activate or not os.path.exists(activate):
+            logger.error('`%s` environment does not exist. Try `pythonbrew venv create %s`.' % (args[1], args[1]))
+            sys.exit(1)
+        
+        self._write("""\
+echo '# Using `%(arg)s` environment (found in %(workon_home)s)'
+echo '# To leave an environment, simply run `deactivate`'
+source '%(activate)s'
+""" % {'arg': args[1], 'workon_home': workon_home, 'activate': activate})
+    
+    def run_command_list(self, options, args):
+        if options.python:
+            pkgname = Package(options.python).name
+            workon_home = os.path.join(PATH_VENVS, pkgname)
+            if pkgname == self._pkgname:
+                logger.log("(*) virtualenv for %(pkgname)s (found in %(workon_home)s)" % {'pkgname': pkgname, 'workon_home': workon_home})
+            else:
+                logger.log("(-) virtualenv for %(pkgname)s (found in %(workon_home)s)" % {'pkgname': pkgname, 'workon_home': workon_home})
+            if os.path.isdir(workon_home):
+                for d in sorted(os.listdir(workon_home)):
+                    if os.path.isdir(os.path.join(workon_home, d)):
+                        logger.log("  %s" % d)
+        else:
+            for pkgname in get_installed_pythons_pkgname():
+                workon_home = os.path.join(PATH_VENVS, pkgname)
+                if os.path.isdir(workon_home):
+                    dirs = os.listdir(workon_home)
+                    if len(dirs) > 0:
+                        if pkgname == self._pkgname:
+                            logger.log("(*) virtualenv for %(pkgname)s (found in %(workon_home)s)" % {'pkgname': pkgname, 'workon_home': workon_home})
+                        else:
+                            logger.log("(-) virtualenv for %(pkgname)s (found in %(workon_home)s)" % {'pkgname': pkgname, 'workon_home': workon_home})
+                        for d in sorted(dirs):
+                            if os.path.isdir(os.path.join(workon_home, d)):
+                                logger.log("  %s" % d)
+    
     def run_command_clone(self, options, args):
         if len(args) < 3:
             logger.error("Unrecognized command line argument: ( 'pythonbrew venv clone <source> <target>' )")
             sys.exit(1)
-        
         if not os.access(PATH_VENVS, os.W_OK):
             logger.error("Can not clone a virtual environment in %s.\nPermission denied." % PATH_VENVS)
             sys.exit(1)
+        if not self._pkgname:
+            logger.error("Unknown python version: ( 'pythonbrew venv clone <source> <target> -p VERSION' )")
+            sys.exit(1)
         
         virtualenv_options = []
         if options.no_site_packages:
@@ -158,31 +205,44 @@ class VenvCommand(Command):
         # Create the new venv first
         self.run_command_create(options, ['create', target])
         
-        print "Copying " + source + "'s libraries to " + target + "'s virtual environment...",
         # Copy all files and folders into the new venv dir, without replacing anything
-        copy_libs(source, target)
-        print "done."
+        self._copy_libs(source, target)
         
         # Activate the new venv
         self.run_command_use(options, ['use', target])
+
+    def run_command_rename(self, options, args):
+        if len(args) < 3:
+            logger.error("Unrecognized command line argument: ( 'pythonbrew venv rename <source> <target>' )")
+            sys.exit(1)
+        if not os.access(PATH_VENVS, os.W_OK):
+            logger.error("Can not rename a virtual environment in %s.\nPermission denied." % PATH_VENVS)
+            sys.exit(1)
+        if not self._pkgname:
+            logger.error("Unknown python version: ( 'pythonbrew venv rename <source> <target> -p VERSION' )")
+            sys.exit(1)
+        
+        source_dir = os.path.join(self._workon_home, args[1])
+        target_dir = os.path.join(self._workon_home, args[2])
+        
+        if not os.path.isdir(source_dir):
+            logger.error('%s does not exist.' % source_dir)
+            sys.exit(1)
+            
+        if os.path.isdir(target_dir):
+            logger.error('Can not overwrite %s.' % target_dir)
+            sys.exit(1)
+        
+        logger.info("Rename `%s` environment to `%s` on %s" % (args[1], args[2], self._workon_home))
+        os.rename(source_dir, target_dir)
     
-    def run_command_delete(self, options, args):
-        for arg in args[1:]:
-            target_dir = os.path.join(self._workon_home, arg)
-            if not os.path.isdir(target_dir):
-                logger.error('%s already does not exist.' % target_dir)
-            else:
-                if not os.access(target_dir, os.W_OK):
-                    logger.error("Can not delete %s.\nPermission denied." % target_dir)
-                    continue
-                logger.info('Deleting `%s` environment in %s' % (arg, self._workon_home))
-                # make command
-                rm_r(target_dir)
-                
     def run_command_print_activate(self, options, args):
         if len(args) < 2:
             logger.error("Unrecognized command line argument: ( 'pythonbrew venv print_activate <project>' )")
             sys.exit(1)
+        if not self._pkgname:
+            logger.error("Unknown python version: ( 'pythonbrew venv print_activate <project> -p VERSION' )")
+            sys.exit(1)
         
         activate = os.path.join(self._workon_home, args[1], 'bin', 'activate')
         if not os.path.exists(activate):
@@ -190,43 +250,41 @@ class VenvCommand(Command):
             sys.exit(1)
             
         logger.log(activate)
-            
     
-    def run_command_use(self, options, args):
-        if len(args) < 2:
-            logger.error("Unrecognized command line argument: ( 'pythonbrew venv use <project>' )")
-            sys.exit(1)
-        
-        activate = os.path.join(self._workon_home, args[1], 'bin', 'activate')
-        if not os.path.exists(activate):
-            logger.error('`%s` environment already does not exist. Try `pythonbrew venv create %s`.' % (args[1], args[1]))
-            sys.exit(1)
-        
-        self._write("""\
-echo '# Using `%(arg)s` environment (found in %(workon_home)s)'
-echo '# To leave an environment, simply run `deactivate`'
-source '%(activate)s'
-""" % {'arg': args[1], 'workon_home': self._workon_home, 'activate': activate})
-        
-    def run_command_list(self, options, args):
-        if options.all:
-            for pkgname in get_installed_pythons_pkgname():
-                workon_home = os.path.join(PATH_VENVS, pkgname)
-                logger.log("# virtualenv for %(pkgname)s (found in %(workon_home)s)" % {'pkgname': pkgname, 'workon_home': workon_home})
-                if os.path.isdir(workon_home):
-                    for d in sorted(os.listdir(workon_home)):
-                        if os.path.isdir(os.path.join(workon_home, d)):
-                            logger.log(d)
-        else:
-            logger.log("# virtualenv for %(pkgname)s (found in %(workon_home)s)" % {'pkgname': self._pkgname, 'workon_home': self._workon_home})
-            if os.path.isdir(self._workon_home):
-                for d in sorted(os.listdir(self._workon_home)):
-                    if os.path.isdir(os.path.join(self._workon_home, d)):
-                        logger.log(d)
+    def _clear(self):
+        self._write("")
     
     def _write(self, src):
         fp = open(PATH_HOME_ETC_VENV, 'w')
         fp.write(src)
         fp.close()
+
+    def _copy_libs(self, source, target):
+        """Copies every lib inside source's site-packages folder into 
+        target's site-packages without replacing already existing libs.
+        source and target are the names of the venvs""" 
+        logger.log("Copying " + source + "'s libraries to " + target + "'s virtual environment...")
+        
+        # File to copy never-the-less (to add libs to the PATH)
+        easy_inst = "easy-install.pth"
+        
+        source_path = glob.glob(os.path.join(PATH_VENVS, self._pkgname, source, "lib", "python*", "site-packages"))[0]
+        target_path = glob.glob(os.path.join(PATH_VENVS, self._pkgname, target, "lib", "python*", "site-packages"))[0]
+        
+        path, dirs, files = os.walk(source_path).next()
+        for curr_dir in dirs:
+            src = os.path.join(source_path, curr_dir)
+            tgt = os.path.join(target_path, curr_dir)
+            if not os.path.exists(tgt):
+                shutil.copytree(src, tgt)
+            
+        for curr_file in files:
+            src = os.path.join(source_path, curr_file)
+            tgt = os.path.join(target_path, curr_file)
+            if not os.path.exists(tgt):
+                shutil.copyfile(src, tgt)
+            elif curr_file == easy_inst:
+                os.remove(tgt)
+                shutil.copyfile(src, tgt)
     
 VenvCommand()
